<?php
/**
 * @package   admintoolswp
 * @copyright Copyright (c)2017-2019 Nicholas K. Dionysopoulos / Akeeba Ltd
 * @license   GNU GPL version 3 or later
 */

use Akeeba\AdminTools\Admin\Helper\Language;
use Akeeba\AdminTools\Admin\Helper\Wordpress;
use Akeeba\AdminTools\Library\Input\Input;
use Akeeba\AdminTools\Library\Uri\Uri;
use Akeeba\Engine\Factory;
use Akeeba\Engine\Platform;
use Akeeba\Engine\Util\Complexify;

define('WPINC', 1);
define('ADMINTOOLSINC', 1);
define('AKEEBAENGINE', 1);
define('AKEEBA_BACKUP_ORIGIN', 'frontend');

if (!defined('ADMINTOOLSWP_PATH'))
{
	define('ADMINTOOLSWP_PATH', __DIR__);
}

require_once ADMINTOOLSWP_PATH . '/helpers/bootstrap.php';
require_once ADMINTOOLSWP_PATH . '/app/engine/Autoloader.php';

// Try our very best to detect WordPress root path.
// We can safely do that since WP will check if this constant is already defined
if (!defined('ABSPATH'))
{
	$root = Wordpress::getSiteRoot();
	define('ABSPATH', $root);
}

// Load the platform
Platform::addPlatform('filescan', ADMINTOOLSWP_PATH . '/app/platform/Filescan');
Platform::getInstance()->load_configuration(1);

$aeconfig = Factory::getConfiguration();
$input    = new Input();
// Unset time limits
$safe_mode = true;

if (function_exists('ini_get'))
{
	$safe_mode = ini_get('safe_mode');
}

if (!$safe_mode && function_exists('set_time_limit'))
{
	@set_time_limit(0);
}

// Is frontend backup enabled and is the Secret Key strong enough??
$febEnabled = Platform::getInstance()->get_platform_configuration_option('frontend_enable', 0) != 0;
$validKey   = Platform::getInstance()->get_platform_configuration_option('frontend_secret_word', '');

if (!Complexify::isStrongEnough($validKey, false))
{
	$febEnabled = false;
}

if (!$febEnabled)
{
	@ob_end_clean();
	echo '403 ' . Language::_('COM_ADMINTOOLS_ERROR_NOT_ENABLED');
	flush();
	exit();
}

// Is the key good?
$key          = $input->get('key', '', 'raw');
$validKeyTrim = trim($validKey);

if (($key != $validKey) || (empty($validKeyTrim)))
{
	@ob_end_clean();
	echo '403 ' . Language::_('COM_ADMINTOOLS_ERROR_INVALID_KEY');
	flush();
	exit();
}

if ($input->get('task', '') != 'step')
{
	Platform::getInstance()->load_configuration(1);
	Factory::resetState();
	Factory::getFactoryStorage()->reset(AKEEBA_BACKUP_ORIGIN);

	$configOverrides['volatile.core.finalization.action_handlers'] = array(
		new \Akeeba\Engine\Finalization\Email()
	);
	$configOverrides['volatile.core.finalization.action_queue'] = array(
		'remove_temp_files',
		'update_statistics',
		'update_filesizes',
		'apply_quotas',
		'send_scan_email'
	);

	// Apply the configuration overrides, please
	$platform = Platform::getInstance();
	$platform->configOverrides = $configOverrides;

	$kettenrad = Factory::getKettenrad();
	$options = array(
		'description' => '',
		'comment'     => '',
		'jpskey'      => ''
	);
	$kettenrad->setup($options);

	Factory::getLog()->open(AKEEBA_BACKUP_ORIGIN);
	Factory::getLog()->log(true, '');

	$kettenrad->tick();
	$kettenrad->tick();

	Factory::saveState(AKEEBA_BACKUP_ORIGIN);

	$array = $kettenrad->getStatusArray();

	try
	{
		Factory::saveState(AKEEBA_BACKUP_ORIGIN);
	}
	catch (\RuntimeException $e)
	{
		$array['Error'] = $e->getMessage();
	}

	if ($array['Error'] != '')
	{
		// An error occurred
		die('500 ERROR -- ' . $array['Error']);
	}

	$noredirect = $input->get('noredirect', 0, 'int');

	if ($noredirect != 0)
	{
		@ob_end_clean();
		header('Content-type: text/plain');
		header('Connection: close');
		echo "301 More work required";
		flush();
		exit();
	}

	$uri = Uri::getInstance();
	$uri->setVar('task', 'step');
	$uri->setVar('key', $input->get('key', '', 'raw'));

	$redirectionUrl = $uri->toString();

	header('HTTP/1.1 302 Found' );
	header('Location: ' . $redirectionUrl);
	header('Content-Type: text/plain');
	header('Connection: close');
	exit();
}

// STEP
Factory::loadState(AKEEBA_BACKUP_ORIGIN);
$kettenrad = Factory::getKettenrad();

$kettenrad->tick();
$array = $kettenrad->getStatusArray();
$kettenrad->resetWarnings(); // So as not to have duplicate warnings reports

try
{
	Factory::saveState(AKEEBA_BACKUP_ORIGIN);
}
catch (\RuntimeException $e)
{
	$array['Error'] = $e->getMessage();
}

if ($array['Error'] != '')
{
	@ob_end_clean();
	echo '500 ERROR -- ' . $array['Error'];
	flush();
	die();
}
elseif ($array['HasRun'] == 1)
{
	// All done
	Factory::nuke();
	Factory::getFactoryStorage()->reset();
	@ob_end_clean();
	header('Content-type: text/plain');
	header('Connection: close');
	echo '200 OK';
	flush();
	exit();
}
else
{
	$noredirect = $input->get('noredirect', 0, 'int');

	if ($noredirect != 0)
	{
		@ob_end_clean();
		header('Content-type: text/plain');
		header('Connection: close');
		echo "301 More work required";
		flush();
		exit();
	}

	$uri = Uri::getInstance();
	$uri->setVar('task', 'step');
	$uri->setVar('key', $input->get('key', '', 'raw'));

	$redirectionUrl = $uri->toString();

	header('HTTP/1.1 302 Found' );
	header('Location: ' . $redirectionUrl);
	header('Content-Type: text/plain');
	header('Connection: close');
	exit();
}
